﻿using System;
using System.Windows.Forms;

namespace RDIFramework.Test
{
    using RDIFramework.Utilities;
    using RDIFramework.WinForm.Utilities;

    /// <summary>
    /// 产品编辑
    /// FrmProductEdit
    /// 
    /// </summary>
    public partial class FrmProductEdit : BaseForm
    {
        private ProductInfoEntity currentProductInfoEntity = null;
        IDbProvider dbProvider = null;

        public FrmProductEdit()
        {
            InitializeComponent();
        }

        public FrmProductEdit(string productId):this()
        {            
            this.EntityId = productId;
        }

        public override void FormOnLoad()
        {
            dbProvider = DbFactoryProvider.GetProvider(DbLinks["RDIFrameworkDBLink"].DbType, SecretHelper.AESDecrypt(DbLinks["RDIFrameworkDBLink"].DbLink));
            BindCategory();
            this.txtProductCode.Text = BusinessLogic.NewGuid();
            //修改数据
            if (!string.IsNullOrEmpty(this.EntityId))
            {                
                BindEditData();
                this.Text = "编辑产品 - " + currentProductInfoEntity.ProductName;
                this.btnSaveContine.Visible = false;
            }
        }

        private void BindCategory()
        {
            BasePageLogic.BindCategory(base.UserInfo, cboProductCategory, "ProductCategory");
        }
            
        #region private void BindEditData() 绑定待修改的数据
        private void BindEditData()
        {
            currentProductInfoEntity = new ProductInfoManager(dbProvider).GetEntity(this.EntityId);
            if (currentProductInfoEntity == null || string.IsNullOrEmpty(currentProductInfoEntity.ProductName)) return;
            //General Information　常规信息
            this.txtProductCode.Text = currentProductInfoEntity.ProductCode;
            this.txtProductName.Text = currentProductInfoEntity.ProductName;
            this.cboProductCategory.SelectedValue = currentProductInfoEntity.ProductCategory;
            this.txtProductModel.Text = currentProductInfoEntity.ProductModel;
            this.txtProductStandard.Text = currentProductInfoEntity.ProductStandard;
            this.txtProductUnit.Text = currentProductInfoEntity.ProductUnit;
            this.txtProductDescription.Text = currentProductInfoEntity.ProductDescription;
                
            //价格信息
            this.txtMiddleRate.Text = currentProductInfoEntity.MiddleRate.ToString();
            this.txtReferenceCoefficient.Text = currentProductInfoEntity.ReferenceCoefficient.ToString();
            this.txtProductPrice.Text = currentProductInfoEntity.ProductPrice.ToString();
            this.txtWholesalePrice.Text = currentProductInfoEntity.WholesalePrice.ToString();
            this.txtPromotionPrice.Text = currentProductInfoEntity.PromotionPrice.ToString();
            this.txtInternalPrice.Text = currentProductInfoEntity.InternalPrice.ToString();
            this.txtSpecialPrice.Text = currentProductInfoEntity.SpecialPrice.ToString();

            //功能描述
            this.txtDescription.Text = currentProductInfoEntity.Description;
            this.chkEnabled.Checked = BusinessLogic.ConvertIntToBoolean(currentProductInfoEntity.Enabled);
        }
        #endregion

        ProductInfoEntity productInfoEntity = null;
        private ProductInfoEntity GetDataEntity()
        {
            productInfoEntity = new ProductInfoEntity
            {
                ProductCode = this.txtProductCode.Text.Trim(),
                ProductName = this.txtProductName.Text.Trim(),
                ProductCategory = BusinessLogic.ConvertToString(this.cboProductCategory.SelectedValue),
                ProductModel = this.txtProductModel.Text.Trim(),
                ProductStandard = this.txtProductStandard.Text.Trim(),
                ProductUnit = this.txtProductUnit.Text.Trim(),
                ProductDescription = this.txtProductDescription.Text.Trim(),
                MiddleRate = BusinessLogic.ConvertToDecimal(this.txtMiddleRate.Text),
                ReferenceCoefficient = BusinessLogic.ConvertToDecimal(this.txtReferenceCoefficient.Text),
                ProductPrice = BusinessLogic.ConvertToDecimal(this.txtProductPrice.Text),
                WholesalePrice = BusinessLogic.ConvertToDecimal(this.txtWholesalePrice.Text),
                PromotionPrice = BusinessLogic.ConvertToDecimal(this.txtPromotionPrice.Text),
                InternalPrice = BusinessLogic.ConvertToDecimal(this.txtInternalPrice.Text),
                SpecialPrice = BusinessLogic.ConvertToDecimal(this.txtSpecialPrice.Text),
                Description = this.txtDescription.Text.Trim(),
                Enabled = chkEnabled.Checked ? 1:0
            };

            return productInfoEntity;
        }

        #region private void SaveData(bool close) 保存新增的数据
        private void SaveData(bool close)
        {            
            // 设置鼠标繁忙状态，并保留原先的状态
            Cursor holdCursor = this.Cursor;
            this.Cursor = Cursors.WaitCursor;
            //读取产品实体数据
            this.GetDataEntity();

            if (new ProductInfoManager(dbProvider).Exists(new string[] { ProductInfoTable.FieldProductCode, ProductInfoTable.FieldProductName, ProductInfoTable.FieldDeleteMark }
                                                        , new string[] { productInfoEntity.ProductCode, productInfoEntity.ProductName, "0" }))
            {
                MessageBoxHelper.ShowWarningMsg("库中已经存在相同数据，请重新检查输入！");
                txtProductName.Focus();
            }
            else
            {
                string returnValue = new ProductInfoManager(dbProvider).AddEntity(productInfoEntity);
                if (returnValue.Trim().Length > 0)
                {
                    this.Changed = true;
                    MessageBoxHelper.ShowSuccessMsg("新增成功！");
                }
                else
                {
                    MessageBoxHelper.ShowWarningMsg("新增数据失败!");
                }
            }

            // 设置鼠标默认状态，原来的光标状态
            this.Cursor = holdCursor;

            if (this.Changed && close)
            {
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
        }
        #endregion

        #region private bool SaveEditData() 保存修改的数据
        /// <summary>
        /// 保存修改的数据
        /// </summary>
        /// <returns></returns>
        private bool SaveEditData()
        {
            // 设置鼠标繁忙状态，并保留原先的状态
            Cursor holdCursor = this.Cursor;
            this.Cursor = Cursors.WaitCursor;   
            bool returnValue = false;
            currentProductInfoEntity.ProductCode = this.txtProductCode.Text.Trim();
            currentProductInfoEntity.ProductName = this.txtProductName.Text.Trim();
            currentProductInfoEntity.ProductCategory = BusinessLogic.ConvertToString(this.cboProductCategory.SelectedValue);
            currentProductInfoEntity.ProductModel = this.txtProductModel.Text.Trim();
            currentProductInfoEntity.ProductStandard = this.txtProductStandard.Text.Trim();
            currentProductInfoEntity.ProductUnit = this.txtProductUnit.Text.Trim();
            currentProductInfoEntity.ProductDescription = this.txtProductDescription.Text.Trim();
            currentProductInfoEntity.MiddleRate = BusinessLogic.ConvertToDecimal(this.txtMiddleRate.Text);
            currentProductInfoEntity.ReferenceCoefficient = BusinessLogic.ConvertToDecimal(this.txtReferenceCoefficient.Text);
            currentProductInfoEntity.ProductPrice = BusinessLogic.ConvertToDecimal(this.txtProductPrice.Text);
            currentProductInfoEntity.WholesalePrice = BusinessLogic.ConvertToDecimal(this.txtWholesalePrice.Text);
            currentProductInfoEntity.PromotionPrice = BusinessLogic.ConvertToDecimal(this.txtPromotionPrice.Text);
            currentProductInfoEntity.InternalPrice = BusinessLogic.ConvertToDecimal(this.txtInternalPrice.Text);
            currentProductInfoEntity.SpecialPrice = BusinessLogic.ConvertToDecimal(this.txtSpecialPrice.Text);
            currentProductInfoEntity.Description = this.txtDescription.Text.Trim();
            currentProductInfoEntity.Enabled = chkEnabled.Checked ? 1: 0;

            string statusMessage = string.Empty;
            returnValue = new ProductInfoManager(dbProvider).Update(currentProductInfoEntity, out statusMessage) > 0;
            if (returnValue)
            {
                this.Changed = true;
                MessageBoxHelper.ShowSuccessMsg(statusMessage);
            }
            else
            {
                MessageBoxHelper.ShowWarningMsg(statusMessage);
            }
           

            // 设置鼠标默认状态，原来的光标状态
            this.Cursor = holdCursor;
            return returnValue;
        }
        #endregion

        #region private bool CheckValueIsEmpty() 界面输入的可用性判断
        private bool CheckValueIsEmpty()
        {
            bool returnValue = true;
            if (string.IsNullOrEmpty(this.txtProductCode.Text) || string.IsNullOrEmpty(this.txtProductName.Text))
            {
                returnValue = false;
                MessageBoxHelper.ShowWarningMsg("名称或编号不能为空！");
            }
            return returnValue;
        }
        #endregion

        private void btnSave_Click(object sender, EventArgs e)
        {
            if(!BasePageLogic.ControlValueIsEmpty(pnlMain))
            {
                return;
            }

            if (string.IsNullOrEmpty(this.EntityId))
            {
                SaveData(true);
            }
            else
            {
                if (SaveEditData())
                {
                    this.DialogResult = DialogResult.OK;
                    this.Close();
                }
            }
        }

        private void btnSaveContine_Click(object sender, EventArgs e)
        {
            if (BasePageLogic.ControlValueIsEmpty(pnlMain))
            {
                SaveData(false);
                BasePageLogic.EmptyControlValue(pnlMain);
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (this.Changed)
            {
                if (MessageBoxHelper.Show("确定放弃操作？") == DialogResult.No)
                {
                    return;
                }

                this.DialogResult = DialogResult.OK;
            }

            this.Close();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            if (this.Changed)
            {
                this.DialogResult = DialogResult.OK;
            }
            this.Close();
        }

        //输入格式控制(主要是对KeyPress事件进行控制，只能输入数据和退格键和小数点)
        private void txtMiddleRate_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar != 8 && !char.IsDigit(e.KeyChar) && e.KeyChar != '.') //小数点的话再加 e.KeyChar != '.'
            {
                e.Handled = true; 
            }     
        }

        private void txtProductPrice_Enter(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(this.txtMiddleRate.Text.Trim()) || string.IsNullOrEmpty(this.txtReferenceCoefficient.Text)) return;
            Decimal? productPrice = BusinessLogic.ConvertToDecimal(this.txtMiddleRate.Text) * BusinessLogic.ConvertToDecimal(this.txtReferenceCoefficient);
            this.txtProductPrice.Text = BusinessLogic.ConvertToString(productPrice);
        }
    }
}
